@@ -76,7 +76,7 @@ startWorkflow <- function(Countries, Species,
                                             'specifyPriors'),
                                    Description = c('---------------','-> Plot data',
                                                    '-> Add structured data',
-                                                   '-> Create an inla.mesh object',
+                                                   '-> Create an fm_mesh_2d object',
                                                    '-> Add data from GBIF',
                                                    '-> Specify sampling region',
                                                    '-> Add spatial covariates',
